(function () {
  if (window.WebViewJavascriptBridge) {
    return;
  }
  var messageHandlers = {};
  var responseCallbacks = {};
  var uniqueId = 1;

  window.WebViewJavascriptBridge = {
    registerHandler: _registerHandler,
    callHandler: _callHandler,
    _handleMessageFromJava: _dispatchMessageFromJava
  };

  function _registerHandler(handlerName, handler) {
    messageHandlers[handlerName] = handler;
  }

  function _callHandler(handlerName, data, responseCallback) {
    if (arguments.length == 2 && typeof data == 'function') {
      responseCallback = data;
      data = null;
    }
    _doSend({
      handlerName: handlerName,
      data: data
    }, responseCallback);
  }

  function _dispatchMessageFromJava(message) {
    setTimeout(_doDispatchMessageFromJava, 0);

    function _doDispatchMessageFromJava() {
      var responseCallback;
      if (message.responseId) {
        responseCallback = responseCallbacks[message.responseId];
        if (!responseCallback) {
          return;
        }
        var msgData = message.responseData;
        if (typeof message.responseData == 'string') {
          try {
            var obj = JSON.parse(message.responseData);
            if (typeof obj == 'object' && obj) {
              msgData = obj;
            }
          } catch (e) {

          }
        }
        responseCallback(msgData);
        delete responseCallbacks[message.responseId];
      } else {
        if (message.callbackId) {
          var callbackResponseId = message.callbackId;
          responseCallback = function (responseData) {
            _doSend({
              handlerName: message.handlerName,
              responseId: callbackResponseId,
              responseData: responseData
            });
          };
        }
        var handler = messageHandlers[message.handlerName];
        if (!handler) {
          console.log("WebViewJavascriptBridge: WARNING: no handler for message from java");
        } else {
          var msgData = message.data
          if (typeof message.data == 'string') {
            try {
              var obj = JSON.parse(message.data);
              if (typeof obj == 'object' && obj) {
                msgData = obj;
              }
            } catch (e) { }
          }
          handler(msgData, responseCallback);
        }
      }
    }
  }

  function _doSend(message, responseCallback) {
    if (responseCallback) {
      var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
      responseCallbacks[callbackId] = responseCallback;
      message['callbackId'] = callbackId;
    }
    try {
      if (window.WVJBInterface) {
        window.WVJBInterface.notice(JSON.stringify(message || {}));
      }
    } catch (e) {

    }
  }

  setTimeout(_callWVJBCallbacks, 0);
  function _callWVJBCallbacks() {
    var callbacks = window.WVJBCallbacks;
    delete window.WVJBCallbacks;
    if (callbacks) {
      for (var i = 0; i < callbacks.length; i++) {
        callbacks[i](window.WebViewJavascriptBridge);
      }
    }
  }
})();